@@ -314,7 +314,37 @@ def _get_all_columns(data):
                 columns.append(name)
     return columns
 
-def insert_sql(data,table_name,schema_name=None):
+def _group_insert_data(data,group_size=500,semicolon=True):
+
+    inserts = []
+    group_size = 500
+    total_rows = len(data)
+    max_groups = int(total_rows / group_size)
+    print(f"    Total rows: {total_rows}")
+    print(f"    max_groups: {max_groups}")
+
+
+    for idx in range(max_groups):
+        start_idx = group_size * idx
+        end_idx = start_idx + group_size
+        if end_idx > len(data):
+            end_idx = len(data)
+        raw_group = data[start_idx:end_idx]
+
+        insert_group = []
+        for dd in raw_group:
+            dd = _csu.unicode_to_html(dd,semicolon)
+            insert_group.append(dd)
+        inserts.append(insert_group)
+        # sql = c.string.sql.insert_sql(insert_group,"cities","idealech_Equari_Meta")
+        # print(f"sql: {sql}")
+
+        # dbm.run("SET foreign_key_checks = 0")
+        # dbm.run(sql)
+        # dbm.run("SET foreign_key_checks = 1")
+    return inserts
+
+def insert_sql(data,table_name,schema_name=None,**kwargs):
     '''
         Generate an insert SQL statement from a list of data.
 
@@ -375,10 +405,26 @@ def insert_sql(data,table_name,schema_name=None):
         `method_name`: gen_insert_sql
         * @TODO []: documentation for gen_insert_sql
     '''
-
+    group_size = _obj.get_kwarg(['group_size'],500,(int),**kwargs)
 
     if isinstance(data,(list)):
-        if len(data) > 0:
+        row_count = len(data)
+        if row_count > 0:
+            if row_count > group_size:
+                # @Mstep [] slice the data into smaller groups.
+                inserts = _group_insert_data(data,group_size)
+                output_sql = ''
+                # @Mstep [LOOP] iterate the sliced data groups
+                for ins in inserts:
+                    # @Mstep [] generate the sql for the group
+                    group_sql = insert_sql(ins)
+                    # @Mstep [IF] if the sql qas successfully generated.
+                    if isinstance(group_sql,(str)):
+                        # @Mstep [] concatenate the group sql to the preceding sql.
+                        output_sql += f"\n\n\n{group_sql}"
+                # @Mstep [RETURN] return the grouped SQL.
+                return output_sql
+
             sql = ''
             value_array = []
             # @Mstep [] get a list of all unique column keys from the data.
@@ -390,6 +436,7 @@ def insert_sql(data,table_name,schema_name=None):
                 value_array.append(_gen_value_string(row,columns))
             # @Mstep [] generate the column string.
             column_string = _gen_column_string(columns)
+            
             # @Mstep [] join the value_array with commas
             value_string = ',\n'.join(value_array)
             # @Mstep [] replace the literal string "__NULL_VALUE__" with null
@@ -410,7 +457,105 @@ def insert_sql(data,table_name,schema_name=None):
 
             # cfu.write.write(path,sql)
             return sql
-    return False
+#     return False
+# def insert_sql(data,table_name,schema_name=None):
+#     '''
+#         Generate an insert SQL statement from a list of data.
+
+#         ----------
+
+#         Arguments
+#         -------------------------
+#         `data` {list}
+#             A list of dictionaries, each representing a row in the table and each key corresponding
+#             to a table column.
+
+#             data = [
+#                 {
+#                     `primary_id`:1,
+#                     `name`:"kitties and titties",
+#                     `timestamp`:None
+#                 }
+#             ]
+
+#         `table_name` {str}
+#             The name of the table to insert the data into.
+
+#         [`schema_name`=None] {str}
+#             The optional schema that the table belongs to.
+
+
+#         Return {str|bool}
+#         ----------------------
+#         The SQL insert statement.
+#         returns False, if the data provided is not a list or the list is empty
+
+
+#         Examples
+#         ----------------------
+#         data = [
+#             {
+#                 "kitty_id":1,
+#                 "name":"kitties and titties",
+#                 "timestamp":123456789,
+#             }
+#         ]
+#         sql = _csu.gen.sql.insert_sql(data,'kitties','titties')
+
+#         ============================== return ==============================
+
+#         INSERT INTO \\`titties\\`.\\`kitties\\` (\\`kitty_id\\`, \\`name\\`, \\`timestamp\\`) VALUES
+
+#         (1, 'kitties and titties', 123456789);
+
+
+
+#         Meta
+#         ----------
+#         `author`: Colemen Atwood
+#         `created`: 06-02-2022 09:13:39
+#         `memberOf`: parse_sql
+#         `version`: 1.0
+#         `method_name`: gen_insert_sql
+#         * @TODO []: documentation for gen_insert_sql
+#     '''
+
+
+#     if isinstance(data,(list)):
+#         if len(data) > 0:
+#             sql = ''
+#             value_array = []
+#             # @Mstep [] get a list of all unique column keys from the data.
+#             columns = _get_all_columns(data)
+
+#             # @Mstep [LOOP] iterate the rows in the data list.
+#             for row in data:
+#                 # @Mstep [] generate the row SQL string and append to the value_array
+#                 value_array.append(_gen_value_string(row,columns))
+#             # @Mstep [] generate the column string.
+#             column_string = _gen_column_string(columns)
+            
+#             # @Mstep [] join the value_array with commas
+#             value_string = ',\n'.join(value_array)
+#             # @Mstep [] replace the literal string "__NULL_VALUE__" with null
+#             value_string = value_string.replace("'__NULL_VALUE__'",'null')
+#             value_string = value_string.replace("__NULL_VALUE__",'null')
+
+#             # @Mstep [] generate the schema string, this is for cases where a table has no schema.
+#             schema_string = ''
+#             # @Mstep [IF] if the schema name was provided.
+#             if schema_name is not None:
+#                 # @Mstep [] generate the schema name string.
+#                 schema_string = f"`{schema_name}`."
+
+#             # @Mstep [] generate the insert statement
+#             sql += f"INSERT INTO {schema_string}`{table_name}` ({column_string}) VALUES \n"
+#             # @Mstep [] concatenate the value_string onto the insert statement.
+#             sql += f"{value_string};"
+
+#             # cfu.write.write(path,sql)
+#             return sql
+#     return False
 
 def sql_type_to_python(value):
     '''
